angular.module('beamng.apps')
.directive('socControl', ['$log', function ($log) {
  return {
    template:
      '<object style="width:100%; height:100%;" type="image/svg+xml" data="/ui/modules/apps/SOC_Control/SOC_control_t01.svg"></object>',
    replace: true,
    restrict: 'EA',
    scope: true,
    link: function (scope, element, attrs) {
      element.on('load', function () {
        let svg = element[0].contentDocument
        let evBtn = angular.element(svg.getElementById('ev_btn'))
        let hevBtn = angular.element(svg.getElementById('hev_btn'))
        let reBtn = angular.element(svg.getElementById('re_btn'))
        let upBtn = angular.element(svg.getElementById('up_btn'))
        let downBtn = angular.element(svg.getElementById('down_btn'))
        let SOC_Txt = svg.getElementById('target_SOC_txt')
        let ev_btn_txt = svg.getElementById('ev_btn_txt')
        let hev_btn_txt = svg.getElementById('hev_btn_txt')
        let re_btn_txt = svg.getElementById('re_btn_txt')
        let state = null
        let offColor = '#949494'
        let onColor = '#7CFC00'
        let incSOC = 5
        let SOC_ChangeDir = 1
        let changeValueIncreaseId

        scope.$on('SettingsChanged', function (event, data) {
          bngApi.activeObjectLua('extensions.socControl.requestState()')
        })

        evBtn.on('click', function () {
          bngApi.activeObjectLua('extensions.socControl.setMode("EV")')
          bngApi.activeObjectLua('extensions.socControl.requestState()')
        })

        hevBtn.on('click', function () {
          bngApi.activeObjectLua('extensions.socControl.setMode("HEV")')
          bngApi.activeObjectLua('extensions.socControl.requestState()')
        })

        reBtn.on('click', function () {
          bngApi.activeObjectLua('extensions.socControl.setMode("RE")')
          bngApi.activeObjectLua('extensions.socControl.requestState()')
        })

        function setNewSOC(newVal) {
          bngApi.activeObjectLua(`extensions.socControl.setSOC(${newVal})`)
          bngApi.activeObjectLua('extensions.socControl.requestState()')
        }

        function changeSOCInc() {
          if(incSOC === 0) return
          setNewSOC(state.targetSOC + (incSOC * SOC_ChangeDir))
        }

        downBtn.on('mousedown', function () {
          incSOC = 5
          SOC_ChangeDir = -1
          changeSOCInc()
          changeValueIncreaseId = setInterval(changeSOCInc, 150)
        })
        downBtn.on('mouseup', function () {
          incSOC = 0
          clearTimeout(changeValueIncreaseId)
        })
        downBtn.on('mouseout', function () {
          incSOC = 0
          clearTimeout(changeValueIncreaseId)
        })

        upBtn.on('mousedown', function () {
          incSOC = 5
          SOC_ChangeDir = 1
          changeSOCInc()
          changeValueIncreaseId = setInterval(changeSOCInc, 150)
        })
        upBtn.on('mouseup', function () {
          incSOC = 0
          clearTimeout(changeValueIncreaseId)
        })
        upBtn.on('mouseout', function () {
          incSOC = 0
          clearTimeout(changeValueIncreaseId)
        })


        scope.$on('SOC_ControlState', function (event, data) {
          scope.$evalAsync(function() {
            //console.log(TAG, 'state:', data)
            state = data
            SOC_Txt.innerHTML = Math.round(state.targetSOC)
            if (state.hybridMode === 'EV') {
              SOC_Txt.style.fill = offColor
              ev_btn_txt.style.fill = onColor
              hev_btn_txt.style.fill = offColor
              re_btn_txt.style.fill = offColor
            } else if (state.hybridMode === 'HEV') {
              SOC_Txt.style.fill = onColor
              ev_btn_txt.style.fill = offColor
              hev_btn_txt.style.fill = onColor
              re_btn_txt.style.fill = offColor
            } else if (state.hybridMode === 'RE') {
              SOC_Txt.style.fill = onColor
              ev_btn_txt.style.fill = offColor
              hev_btn_txt.style.fill = offColor
              re_btn_txt.style.fill = onColor
            }
          })
        })

        scope.$on('VehicleFocusChanged', function () {
          bngApi.activeObjectLua('extensions.socControl.requestState()')
        })

        bngApi.engineLua('settings.notifyUI()')
      })
    }
  }
}])
